TotalThreat = LibStub("AceAddon-3.0"):NewAddon("TotalThreat", "AceConsole-3.0", "AceEvent-3.0","AceTimer-3.0")

-- Local versions for performance
local tinsert, tremove, tgetn = table.insert, table.remove, table.getn
local pairs, ipairs = pairs, ipairs

TotalThreat.unitname = nil
TotalThreat.threatvalue = nil
TotalThreat.startime = nil

local maxFights = 10
local fights = {}

local GREEN = "|cff00ff00"
local YELLOW = "|cffffff00"
local BLUE = "|cff0198e1"
local ORANGE = "|cffff9933"

local LDB = LibStub("LibDataBroker-1.1")
local LibQTip = LibStub('LibQTip-1.0')

local Broker = CreateFrame("Frame")

Broker.obj = LDB:NewDataObject("Total Threat", {
    type = "data source",
    icon = "Interface\\Icons\\Ability_DualWield",
    label = "Total Threat",
    text = "Total Threat",
    barValue = 0,
    barR = 0,
    barG = 0,
    barB = 1,
} )

function Broker.obj:OnEnter()
	local tooltip = LibQTip:Acquire("TotalThreatTooltip", 4, "LEFT", "CENTER", "CENTER", "CENTER")
	self.tooltip = tooltip 

	tooltip:AddHeader(GREEN..'Unit Name', 
		YELLOW..'Threat', 
		BLUE..'Time', 
		ORANGE..'TPS')
	tooltip:AddSeparator(1)
	
	if tgetn(fights) == 0 then
		tooltip:AddLine('No fight data')
	else
		for i = 1, tgetn(fights) do
			local fight = fights[i]
			if fight then
				tooltip:AddLine(GREEN..fight.unitname, 
					YELLOW..fight.threat, 
					BLUE..fight.totaltime, 
					ORANGE..fight.tps)
			end
		end	
	end

	tooltip:SmartAnchorTo(self)
	tooltip:Show()
end

function Broker.obj:OnLeave()
	LibQTip:Release(self.tooltip)
	self.tooltip = nil
end

local defaults = {
    profile = {
        verbose = true
    }
}

local options

function TotalThreat:GetOptions()
    if not options then
        options = {
            name = "TotalThreat",
            handler = TotalThreat,
            type = 'group',
            args = {
                verbose = {
                    name = "Verbose",
                    desc = "Toggles the display of informational messages",
                    type = "toggle",
                    set = function(info, val) self.db.profile.verbose = val end,
                    get = function(info) return self.db.profile.verbose end,
                }
            }
        }
    end
    
    return options
end

function TotalThreat:ChatCommand(input)
    if not input or input:trim() == "" then
        InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
    else
        LibStub("AceConfigCmd-3.0").HandleCommand(TotalThreat, "acl", "TotalThreat", input)
    end
end

function TotalThreat:OnInitialize()
    -- Load the settings
    self.db = LibStub("AceDB-3.0"):New("TotalThreatDB", defaults, "Default")

    -- Register the options table
    LibStub("AceConfig-3.0"):RegisterOptionsTable("TotalThreat", self:GetOptions())
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(
	    "TotalThreat", ADDON_NAME)

    -- Register to receive the PLAYER_REGEN_ENABLED event
    self:RegisterEvent("PLAYER_REGEN_ENABLED")

    -- Register to receive the PLAYER_REGEN_DISABLED event
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
end

function TotalThreat:OnEnable()
    -- Register to receive the PLAYER_REGEN_ENABLED event
    self:RegisterEvent("PLAYER_REGEN_ENABLED")

    -- Register to receive the PLAYER_REGEN_DISABLED event
    self:RegisterEvent("PLAYER_REGEN_DISABLED")
end

function TotalThreat:OnDisable()
    -- Unregister events
    self:UnregisterEvent("PLAYER_REGEN_ENABLED")
    self:UnregisterEvent("PLAYER_REGEN_DISABLED")
end

function TotalThreat:UpdateDisplay()
	if UnitExists("target") and not UnitIsFriend("player", "target") then
	    local name = UnitName("target")
		local isTanking, status, threatpct, rawthreatpct, threatvalue =
			UnitDetailedThreatSituation("player", "target")
		if threatvalue ~= nil and threatvalue > 0 then
			TotalThreat.unitname = name
			TotalThreat.threatvalue = threatvalue
		end
	end
end

local function format_threatvalue(value)
	if value == nil then
		return "0"
	elseif value >= 100000 then
		return ("%2.1fk"):format(value / 100000)
	else
		return ("%d"):format(value / 100)
	end
end

function TotalThreat:AddFightData(unitname, threat, totaltime, tps)
	local data = {}
	data.unitname = unitname
	data.threat = threat
	data.totaltime = totaltime
	data.tps = tps
	tinsert(fights, data)
	
	while tgetn(fights) > maxFights do
		tremove(fights, 1)
	end
end

function TotalThreat:PLAYER_REGEN_ENABLED()
	-- Called when combat ends.  Output threat information.
	if (TotalThreat.unitname == nil) then
		self:Print("No valid unit present.")
	elseif (TotalThreat.threatvalue == nil) then
		self:Print("No threat value.")
	else
		local totaltime = time() - TotalThreat.starttime
		local formattedThreat = format_threatvalue(TotalThreat.threatvalue)
		local tps = format_threatvalue(TotalThreat.threatvalue / math.max(1,totaltime))

		if (self.db.profile.verbose) then
		    local reportFormat = "%s (%s, %d, %s) (%d)"
			self:Print(
			    reportFormat:format(
			        TotalThreat.unitname, 
			        formattedThreat, 
			        totaltime, 
			        tps, 
			        TotalThreat.threatvalue))
		end

		-- Update our table of fight data
		self:AddFightData(TotalThreat.unitname, formattedThreat, totaltime, tps)
	end
end

function TotalThreat:PLAYER_REGEN_DISABLED()
	TotalThreat.starttime = time()
	-- Called when combat starts.  Create a timer to fetch the current threat.
	update_timer = self:ScheduleRepeatingTimer("UpdateDisplay", 0.5)
end
